/*
** File: prodtest.c  
** Project: ADB3 core driver
** Purpose: OS-independent IOCTL handlers to support production testing.
**
** (C) Copyright Alpha Data 2011
*/

#include <df.h>
#include "device.h"
#include "prodtest.h"

DfIoStatus
ioctlGetProdTestInfo(
	Adb3CoreDeviceContext* pDevCtx,
  void* pBuffer,
  unsigned int outSize)
{
  IOCTLS_ADB3_GETPRODTESTINFO* pIoctl = (IOCTLS_ADB3_GETPRODTESTINFO*)pBuffer;
  DfIoStatus status = DfIoStatusSuccess;

  if (NULL == pIoctl || sizeof(pIoctl->out) != outSize) {
    return DfIoStatusInvalid;
  }

  dfZeroMemory(&pIoctl->out, sizeof(pIoctl->out));
  pIoctl->out.serialNumber = pDevCtx->info.prodTest.serialNumber;
  pIoctl->out.cpldRevision = pDevCtx->info.prodTest.cpldRevision;
  pIoctl->out.pcbRevision = pDevCtx->info.prodTest.pcbRevision;
  pIoctl->out.bridgeRevision = pDevCtx->pDevObj->hardwareId.variant.pci.revision;
  pIoctl->out.bridgeDate = pDevCtx->info.prodTest.bridgeDate;
  pIoctl->out.bridgeTime = pDevCtx->info.prodTest.bridgeTime;

  return status;
}

static boolean_t
serviceInterrupts(
	Adb3CoreDeviceContext* pDevCtx)
{
  return pDevCtx->methods.pIsr(&pDevCtx->interrupt.object, pDevCtx);
}

DfIoStatus
ioctlServiceIrq(
	Adb3CoreDeviceContext* pDevCtx,
  Adb3CoreClientContext* pClCtx,
  void* pBuffer,
  unsigned int outSize)
{
  IOCTLS_ADB3_SERVICEIRQ* pIoctl = (IOCTLS_ADB3_SERVICEIRQ*)pBuffer;
  DfIoStatus status = DfIoStatusSuccess;

  if (NULL == pIoctl || sizeof(pIoctl->out) != outSize) {
    return DfIoStatusInvalid;
  }
  if (!pClCtx->bPrivileged) {
    return DfIoStatusError(ADB3_ACCESS_DENIED);
  }
  if (!pDevCtx->interrupt.bPolledMode) {
    return DfIoStatusError(ADB3_WRONG_MODE);
  }

  dfZeroMemory(&pIoctl->out, sizeof(pIoctl->out));
  if (pDevCtx->interrupt.bConnected) {
    serviceInterrupts(pDevCtx);
  }

  return status;
}
